#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// Structure représentant une image RAW
typedef unsigned char UCHAR;
struct IMAGERAW {
    int hauteur;
    int largeur;
    UCHAR* image;
};
typedef struct IMAGERAW IMAGE;

// Fonction de lecture d'une image RAW à partir d'un fichier
// Titre : lire_image
// Description : Lit une image à partir d'un fichier au format RAW.
// Paramètres : nom_fichier, qui est le nom du fichier à lire (chaîne de caractères)
// Retourne un pointeur vers la structure IMAGE contenant les informations de l'image lue.
IMAGE* lire_image(const char* nom_fichier) {
    FILE *fichier = fopen(nom_fichier, "rb");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        //exit(EXIT_FAILURE);
    }


    IMAGE* image = (IMAGE*)malloc(sizeof(IMAGE));
    if (image == NULL) {
        perror("Erreur lors de l'allocation de mémoire");
        //exit(EXIT_FAILURE);
    }

    // Demande à l'utilisateur d'entrer les dimensions de l'image
    printf("Entrez la hauteur de l\'image\n");
    scanf("%d", &image->hauteur);
    printf("Entrez la largeur de l\'image\n");
    scanf("%d", &image->largeur);

    // Allouer de la mémoire pour l'image
    image->image = (UCHAR*)malloc(image->hauteur * image->largeur);
    if (image->image == NULL) {
        perror("Erreur lors de l'allocation de mémoire");
        //exit(EXIT_FAILURE);
    }

    // Lire les pixels de l'image
    fread(image->image, sizeof(UCHAR), static_cast<size_t>(image->hauteur) * image->largeur, fichier);

    fclose(fichier);
    return image;
}

// Fonction de cryptage/décryptage d'une image par XOR
// Titre : crypter_decrypter
// Description : Crypte ou décrypte une image en utilisant l'opération XOR avec une clé de 8 bits.
// Paramètre : imageS, qui est un pointeur vers la structure IMAGE représentant l'image source (adresse)
// Retourne un pointeur vers la structure IMAGE contenant l'image résultante après le cryptage/décryptage.
IMAGE* crypter_decrypter(IMAGE* imageS) {
    IMAGE* imageD = (IMAGE*)malloc(sizeof(IMAGE));
    if (imageD == NULL) {
        perror("Erreur lors de l'allocation de mémoire");
        //exit(EXIT_FAILURE);
    }

    // Copier les dimensions de l'image source
    imageD->hauteur = imageS->hauteur;
    imageD->largeur = imageS->largeur;

    // Allouer de la mémoire pour l'image destination
    imageD->image = (UCHAR*)malloc(imageD->hauteur * imageD->largeur);
    if (imageD->image == NULL) {
        perror("Erreur lors de l'allocation de mémoire ");
        //exit(EXIT_FAILURE);
    }

    // Demander à l'utilisateur si le calcul se fait par rangée ou par colonne
    char choix;
    printf("Crypter/Décrypter par (r)angée ou (c)olonne ?");
    scanf("%c", &choix);

    // Enregistrer le temps de début
    clock_t debut = clock();

    // Effectuer le cryptage/décryptage
    for (int i = 0; i < imageD->hauteur; i++) {
        for (int j = 0; j < imageD->largeur; j++) {
            // Clé de 8 bits (peut être modifiée)
            UCHAR cle = 0x00;

            if (choix == 'r') {
                // Cryptage par rangée
                int index = i * imageD->largeur + j;
                imageD->image[index] = imageS->image[index] ^ cle;
            }
            else if (choix == 'c') {
                // Cryptage par colonne
                int index = j * imageD->hauteur + i;
                imageD->image[index] = imageS->image[index] ^ cle;
            }
        }
    }


    // Enregistrer le temps de fin
    clock_t fin = clock();

    // Calculer et afficher le temps d'exécution
    double temps_execution = ((double)(fin - debut)) / CLOCKS_PER_SEC;
    printf("Temps d'exécution : %f secondes\n", temps_execution);

    return imageD;
}

// Fonction d'écriture d'une image cryptée/décryptée dans un fichier
// Titre : ecrire_image
// Description : Écrit une image cryptée ou décryptée dans un fichier au format RAW.
// Paramètres :
//   - image : Pointeur vers la structure IMAGE représentant l'image à écrire (adresse)
//   - nom_fichier : Nom du fichier dans lequel écrire l'image (chaîne de caractères)
// Ne retourne pas de valeur ou d'information
void ecrire_image(IMAGE* image, const char* nom_fichier) {
    FILE* fichier = fopen(nom_fichier, "wb");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        //exit(EXIT_FAILURE);
    }

    // Écrire les dimensions de l'image
    fwrite(&image->hauteur, sizeof(int), 1, fichier);
    fwrite(&image->largeur, sizeof(int), 1, fichier);

    // Écrire les pixels de l'image
    fwrite(image->image, sizeof(UCHAR), image->hauteur * image->largeur, fichier);

    fclose(fichier);
}

// Alexis Ossene, OSSA81360300, groupe 01
// La fonction main fait appelle aux différentes fonctions définies dans le programme afin de lire l'image "benji.raw" avec la fonction lire_image, 
// de crypter/décrypter cette image avec la fonction crypter_decrypter, 
// et de créer une nouvelle image appellée "image_cryptee.raw".
// On fini par libérer la mémoire allouée au programme.
int main() {
    // Lire l'image à partir d'un fichier
    IMAGE* imageSource = lire_image("benji.raw");

    // Crypter/décrypter l'image
    IMAGE* imageDestination = crypter_decrypter(imageSource);

    // Écrire l'image cryptée/décryptée dans un fichier
    ecrire_image(imageDestination, "image_cryptee.raw");

    // Libérer la mémoire
    free(imageSource->image);
    free(imageSource);

    free(imageDestination->image);
    free(imageDestination);

    return 0;
}
