/*
  Programme réalisé par :
  - Rayane Dakech DAKR70260500
  - Mohamed Tazi TAZM76370200
  - Tameze Voufo Manuelle Tatiana TAMM78350000
  
  Groupe 01 du Mercredi de 15:30-18:30.
*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>
#include <time.h>

// Définition d'un type pour représenter un octet (byte)
typedef unsigned char UCHAR;

// Structure pour représenter une image brute avec sa hauteur, sa largeur et ses pixels
struct IMAGERAW
{
    int h;      // Hauteur de l'image
    int l;      // Largeur de l'image
    UCHAR *image; // Tableau d'octets représentant les pixels de l'image
};

// Alias pour la structure IMAGERAW
typedef struct IMAGERAW IMAGE;

// Fonction pour lire une image à partir d'un fichier
IMAGE *lire_image()
{
    char nomfichier[80]; // Nom du fichier image
    IMAGE *im;           // Pointeur vers la structure représentant l'image
    FILE *fpI;           // Pointeur de fichier pour le fichier image

    // Demander le nom de fichier à l'utilisateur
    printf("Entrez le nom du fichier : ");
    scanf("%s", nomfichier);

    // Ouvrir le fichier en lecture binaire et valider
    fpI = fopen(nomfichier, "rb");
    if (fpI == NULL)
    {
        perror("Erreur lors de l'ouverture du fichier");
        exit(EXIT_FAILURE);
    }

    // Demander et valider la hauteur et la largeur de l’image
    int l, h;
    do
    {
        printf("Entrez la largeur de l'image : ");
        scanf("%d", &l);
        if (l <= 0)
        {
            printf("La largeur doit être un entier positif.\n");
        }
    } while (l <= 0);

    do
    {
        printf("Entrez la hauteur de l'image : ");
        scanf("%d", &h);
        if (h <= 0)
        {
            printf("La hauteur doit être un entier positif.\n");
        }
    } while (h <= 0);

    // Allouer l'espace mémoire pour la structure représentant l'image
    im = (IMAGE *)malloc(sizeof(IMAGE));

    // Initialiser les champs de la structure image
    im->l = l;
    im->h = h;
    im->image = (UCHAR *)malloc(im->h * im->l * sizeof(UCHAR));

    // Lire l’image depuis le disque dans le tableau de pixels
    size_t im_lus = fread(im->image, sizeof(UCHAR), im->h * im->l, fpI);
    if (im_lus != (size_t)(im->h * im->l))
    {
        perror("Erreur lors de la lecture de l'image depuis le fichier");
        fclose(fpI);
        free(im->image);
        free(im);
        exit(EXIT_FAILURE);
    }

    // Fermer le fichier
    fclose(fpI);

    // Retourner le pointeur vers la structure représentant l'image
    return im;
}


IMAGE *crypter_decrypter(IMAGE *im)
{
    int m, n;               // Variables pour les indices de boucle
    UCHAR cle;               // Clé de chiffrement
    IMAGE *im_resultas;        // Pointeur vers la structure résultante
    struct timeb start, finish;  // Structures pour mesurer le temps
    unsigned long duration;  // Durée d'exécution en millisecondes

    // Allouer l'espace mémoire pour la structure résultante
    im_resultas = (IMAGE *)malloc(sizeof(IMAGE));

    // Vérifier si l'allocation mémoire a réussi
    if (im_resultas == NULL)
    {
        printf("Allocation mémoire échouée");
        exit(EXIT_FAILURE);
    }

    // Initialiser les champs de la structure résultante avec les mêmes dimensions que l'image d'entrée
    im_resultas->h = im->h;
    im_resultas->l = im->l;

    // Allouer l'espace mémoire pour les pixels de l'image résultante
    im_resultas->image = (UCHAR *)malloc(im_resultas->h * im_resultas->l * sizeof(UCHAR));

    // Vérifier si l'allocation mémoire a réussi
    if (im_resultas->image == NULL)
    {
        printf("Allocation mémoire échouée");
        exit(EXIT_FAILURE);
    }

    // Demander à l'utilisateur d'entrer la clé de chiffrement
    printf("Entrez la clé de chiffrement (0-255): ");
    scanf("%hhu", &cle);

    // Demander à l'utilisateur de choisir l'ordre de parcours (rangée ou colonne)
    printf("Entrez l'ordre de parcours (1 pour rangée, 2 pour colonne): ");
    int choix_parcours;
    scanf("%d", &choix_parcours);

    // Mesurer le temps de début de l'opération
    ftime(&start);

    // Chiffrement ou déchiffrement en fonction du choix de parcours
    if (choix_parcours == 1)
    {
        // Parcourir les rangées de l'image
        for (m = 0; m < im->h; m++)
        {
            // Parcourir les colonnes de l'image
            for (n = 0; n < im->l; n++)
            {
                // Chiffrer ou déchiffrer chaque pixel avec la clé
                im_resultas->image[m * im->l + n] = im->image[m * im->l + n] ^ cle;
            }
        }
    }
    else if (choix_parcours == 2)
    {
        // Parcourir les colonnes de l'image
        for (n = 0; n < im->l; n++)
        {
            // Parcourir les rangées de l'image
            for (m = 0; m < im->h; m++)
            {
                // Chiffrer ou déchiffrer chaque pixel avec la clé
                im_resultas->image[m * im->l + n] = im->image[m * im->l + n] ^ cle;
            }
        }
    }
    else
    {
        // Choix de parcours invalide
        printf("Choix de parcours invalide\n");
        exit(EXIT_FAILURE);
    }

    // Mesurer le temps de fin de l'opération
    ftime(&finish);

    // Calculer la durée totale de l'opération
    duration = ((unsigned long)finish.time * 1000L + (unsigned long)finish.millitm) - ((unsigned long)start.time * 1000L + (unsigned long)start.millitm);

    // Afficher le temps d'exécution
    printf("Temps d'execution : %lu ms\n", duration);
    // Retourner l'image résultante
    return im_resultas;
}

void ecrire_image(IMAGE *imageD)
{
    char nomfichier[80]; // Nom du fichier de destination
    FILE *fpD;           // Pointeur de fichier pour le fichier de destination

    // Demander à l'utilisateur d'entrer le nom du fichier de destination avec l'extension .raw
    printf("Entrez le nom du fichier de destination (.raw): ");
    scanf("%s", nomfichier);

    // Ouvrir le fichier de destination en écriture binaire et valider
    fpD = fopen(nomfichier, "wb");
    if (fpD == NULL)
    {
        perror("Erreur lors de l'ouverture du fichier");
        exit(EXIT_FAILURE);
    }

    // Écrire les pixels de l'image dans le fichier
    size_t im_ecrit = fwrite(imageD->image, sizeof(UCHAR), imageD->h * imageD->l, fpD);

    // Vérifier si l'écriture a réussi pour tous les éléments
    if (im_ecrit != (size_t)(imageD->h * imageD->l))
    {
        fprintf(stderr, "Erreur lors de l'écriture de l'image\n");
        fclose(fpD);
        exit(EXIT_FAILURE);
    }

    // Fermer le fichier de destination
    fclose(fpD);
}


int main()
{
    IMAGE *im; // Pointeur vers la structure représentant l'image
    IMAGE *im_resultas; // Pointeur vers l'image cryptée
    // LECTURE DE L’IMAGE À CRYPTER
    im = lire_image(); // Appel de la fonction lire_image pour lire une image depuis un fichier

    // CRYPTAGE DE L’IMAGE
    im_resultas=crypter_decrypter(im); // Appel de la fonction crypter_decrypter pour crypter l'image

    // SAUVEGARDE DE L’IMAGE CRYPTEE
    ecrire_image(im_resultas); // Appel de la fonction ecrire_image pour sauvegarder l'image cryptée dans un fichier

    // LIBÉRER LA RAM ATTRIBUÉE DYNAMIQUEMENT POUR L'IMAGE ORIGINALE
    free(im->image); // Libération de la mémoire allouée pour les pixels de l'image originale
    free(im);        // Libération de la mémoire allouée pour la structure représentant l'image originale

    return 0; // Indique une sortie réussie du programme
}

