#include <stdio.h>
#include <stdlib.h>
#include <time.h>

//Définition des types et des structures

typedef unsigned char UCHAR;

struct IMAGERAW {
    int hauteur;
    int largeur;
    UCHAR *image;
};

typedef struct IMAGERAW IMAGE;

//Fonction permettant de lire une image depuis un fichier .raw

IMAGE *lire_image(const char *nom_fichier) {
    FILE *fichier = fopen(nom_fichier, "rb");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        exit(EXIT_FAILURE);
    }

    //Lire les dimensions de l'image

    IMAGE *image = (IMAGE *)malloc(sizeof(IMAGE));
    fread(&image->hauteur, sizeof(int), 1, fichier);
    fread(&image->largeur, sizeof(int), 1, fichier);

    //Allouer de la mémoire pour l'image

    image->image = (UCHAR *)malloc(image->hauteur * image->largeur * sizeof(UCHAR));

    //Lire les pixels de l'image

    fread(image->image, sizeof(UCHAR), image->hauteur * image->largeur, fichier);

    //Fermer le fichier

    fclose(fichier);

    return image;
}

//Fonction pour crypter ou décrypter une image avec une clé XOR

IMAGE *crypter_decrypter(IMAGE *imageS, int choix) {
    IMAGE *imageD = (IMAGE *)malloc(sizeof(IMAGE));
    imageD->hauteur = imageS->hauteur;
    imageD->largeur = imageS->largeur;
    imageD->image = (UCHAR *)malloc(imageD->hauteur * imageD->largeur * sizeof(UCHAR));

    int i, j;
    UCHAR cle = 0x55;  //Clé de 8 bits (peut être changée)

    clock_t debut, fin;
    double temps_execution;

    debut = clock();

    if (choix == 1) {

        //Cryptage rangée

        for (i = 0; i < imageS->hauteur; i++) {
            for (j = 0; j < imageS->largeur; j++) {
                imageD->image[i * imageS->largeur + j] = imageS->image[i * imageS->largeur + j] ^ cle;
            }
        }
    } else if (choix == 2) {

        //Cryptage colonne

        for (j = 0; j < imageS->largeur; j++) {
            for (i = 0; i < imageS->hauteur; i++) {
                imageD->image[i * imageS->largeur + j] = imageS->image[i * imageS->largeur + j] ^ cle;
            }
        }
    } else {
        printf("Choix non valide.\n");
        exit(EXIT_FAILURE);
    }

    fin = clock();
    temps_execution = ((double)(fin - debut)) / CLOCKS_PER_SEC;
    printf("Temps d'exécution : %.6f secondes\n", temps_execution);

    return imageD;
}

//Fonction pour écrire une image dans un fichier .raw

void ecrire_image(IMAGE *imageD, const char *nom_fichier) {
    FILE *fichier = fopen(nom_fichier, "wb");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        exit(EXIT_FAILURE);
    }

    //Dimensions de l'image

    fwrite(&imageD->hauteur, sizeof(int), 1, fichier);
    fwrite(&imageD->largeur, sizeof(int), 1, fichier);

    //Pixels de l'image

    fwrite(imageD->image, sizeof(UCHAR), imageD->hauteur * imageD->largeur, fichier);

    //Fermer le fichier

    fclose(fichier);
}

int main() { //Oumar Al Hassane DIA - Code permanent : DIAO90320400 

    //Demander le nom du fichier à l'utilisateur

    char nom_fichier[256];
    printf("Entrez le nom du fichier .raw : ");
    scanf("%s", nom_fichier);

    //Lecture de l'image

    IMAGE *imageSource = lire_image(nom_fichier);

    //Demande de choix à l'utilisateur

    int choix;
    printf("Choisissez le mode de cryptage/décryptage (1 pour par rangée, 2 pour par colonne) : ");
    scanf("%d", &choix);

    //Cryptage et Décryptage

    IMAGE *imageDestination = crypter_decrypter(imageSource, choix);

    //Écriture de l'image cryptée ou décryptée

    ecrire_image(imageDestination, "image_cryptee.raw");

    //Libération de la mémoire

    free(imageSource->image);
    free(imageSource);
    free(imageDestination->image);
    free(imageDestination);

    return 0;
}
