//________________________________________________________________________________________________________________________//
//     Auteurs : Youma Koumé, Anta Diack
//     Codes permanents : KOUY67310400  DIAA75260400
//     Description : Programme qui effectue le cryptage/décryptage de fichiers image .raw
//     Groupe 01 : Mercredi
//  _____________________________________________________________________________________________________________________//

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// Stockage des informations de l'image

typedef unsigned char UCHAR;

struct IMAGERAW {
    int hauteur;
    int largeur;
    UCHAR *image;
};

typedef struct IMAGERAW IMAGE;

// Fonction de lecture de l'image à crypter ou décrypter

IMAGE *lire_image(const char *nom_fichier, int hauteur, int largeur) {
    FILE *fichier = fopen(nom_fichier, "rb");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        return NULL;
    }

    IMAGE *image = (IMAGE *)malloc(sizeof(IMAGE));
    if (image == NULL) {
        fclose(fichier);
        perror("Erreur d'allocation mémoire");
        return NULL;
    }

    image->hauteur = hauteur;
    image->largeur = largeur;
    image->image = (UCHAR *)malloc(hauteur * largeur * sizeof(UCHAR));
    if (image->image == NULL) {
        fclose(fichier);
        free(image);
        perror("Erreur d'allocation mémoire pour l'image");
        return NULL;
    }

    fread(image->image, sizeof(UCHAR), hauteur * largeur, fichier);
    fclose(fichier);

    return image;
}

// Fonction qui permet la sauvegarde de l'image cryptée ou décryptée

void ecrire_image(const char *nom_fichier, IMAGE *image) {
    FILE *fichier = fopen(nom_fichier, "wb");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier de sortie");
        return;
    }

    fwrite(image->image, sizeof(UCHAR), image->hauteur * image->largeur, fichier);
    fclose(fichier);
}

// Fonction permettant le cryptage et le décryptage d'image avec une fonction de codage XOR

IMAGE *crypter_decrypter(IMAGE *imageS) {
    IMAGE *imageD = (IMAGE *)malloc(sizeof(IMAGE));
    if (imageD == NULL) {
        perror("Erreur d'allocation mémoire pour l'image cryptée/décryptée");
        return NULL;
    }

    imageD->hauteur = imageS->hauteur;
    imageD->largeur = imageS->largeur;
    imageD->image = (UCHAR *)malloc(imageS->hauteur * imageS->largeur * sizeof(UCHAR));
    if (imageD->image == NULL) {
        free(imageD);
        perror("Erreur d'allocation mémoire pour l'image cryptée/décryptée");
        return NULL;
    }

    // Algorithme de cryptage XOR
    UCHAR cle = 255 ; // Clé de 8 bits 
    int i;
    for (i = 0; i < imageS->hauteur * imageS->largeur; i++) {
        imageD->image[i] = imageS->image[i] ^ cle; // Opération XOR avec la clé
    }

    return imageD;
}

// fonction principale du programme

int main() {
    const char *nom_fichier_entree = "benji.raw";
    const char *nom_fichier_sortie = "benji_sortie.raw";
    int hauteur = 1409;
    int largeur = 1690;

    IMAGE *imageA = lire_image(nom_fichier_entree, hauteur, largeur);
    if (imageA == NULL) {
        return 1;
    }

    IMAGE *imageB = crypter_decrypter(imageA);
    if (imageB == NULL) {
        free(imageA->image);
        free(imageA);
        return 1;
    }

    ecrire_image(nom_fichier_sortie, imageB);

    free(imageA->image);
    free(imageA);
    free(imageB->image);
    free(imageB);

    return 0;
}
