/* Auteur :
groupe 01 Mercredi
Serigne Fallou ngom
Diariatou sow
Djiaha Kouega Marcy Audrey
Chamko Itendji Victoire Shiphra
Description du Programme :
Le programme dvelopp en langage C offre la possibilit
de crypter ou dcrypter des
fichiers image au format .raw en utilisant l'algorithme de codage XOR. L'application se base
sur une structure de donnes ddie, IMAGERAW, pour reprsenter les caractristiques d'une image, telles que la hauteur,
la largeur et les donnes d'image. Le cryptage ou dcryptage est effectu en utilisant une cl de 8 bits, et le programme permet  l'utilisateur de choisir si
l'opration doit tre ralise par range ou par colonne.
*/
#include <stdio.h>
#include <stdlib.h>

typedef unsigned char UCHAR;

struct IMAGERAW {
    int hauteur;
    int largeur;
    UCHAR *image;
};

typedef struct IMAGERAW IMAGE;

// Fonction pour lire une image  partir d'un fichier .raw
// Paramtres :
//   - filename : nom du fichier  lire
// Retourne un pointeur vers une structure IMAGE contenant les donnes de l'image lue
IMAGE *lire_image(char *filename) {
    // Ouvrir le fichier en mode lecture binaire
    FILE *file = fopen(filename, "rb");
    if (file == NULL) {
        perror("Impossible d'ouvrir le fichier");
        return NULL;
    }

    // Allouer de la mmoire pour la structure IMAGE
    IMAGE *image = malloc(sizeof(IMAGE));
    if (image == NULL) {
        perror("Impossible d'allouer de la mmoire");
        fclose(file);
        return NULL;
    }

    // Lire les donnes de l'image
    fread(&(image->hauteur), sizeof(int), 1, file);
    fread(&(image->largeur), sizeof(int), 1, file);

    // Allouer de la mmoire pour les donnes de l'image
    image->image = malloc(sizeof(UCHAR) * image->hauteur * image->largeur);
    if (image->image == NULL) {
        perror("Impossible d'allouer de la mmoire pour les donnes de l'image");
        free(image);
        fclose(file);
        return NULL;
    }

    fread(image->image, sizeof(UCHAR), image->hauteur * image->largeur, file);

    // Fermer le fichier
    fclose(file);

    return image;
}

// Fonction pour crypter ou dcrypter une image en utilisant un XOR avec une cl de 8 bits
// Paramtres :
//   - imageS : pointeur vers la structure IMAGE de l'image source  crypter ou dcrypter
//   - key : cl de 8 bits utilise pour l'opration XOR
// Retourne un pointeur vers une nouvelle structure IMAGE contenant l'image crypte ou dcrypte
IMAGE *crypter_decrypter(IMAGE *imageS, UCHAR key) {
    // Allouer de la mmoire pour la nouvelle image
    IMAGE *imageD = malloc(sizeof(IMAGE));
    if (imageD == NULL) {
        perror("Impossible d'allouer de la mmoire pour la nouvelle image");
        return NULL;
    }

    // Copier les donnes de l'image source
    imageD->hauteur = imageS->hauteur;
    imageD->largeur = imageS->largeur;

    // Allouer de la mmoire pour les donnes de la nouvelle image
    imageD->image = malloc(sizeof(UCHAR) * imageD->hauteur * imageD->largeur);
    if (imageD->image == NULL) {
        perror("Impossible d'allouer de la mmoire pour les donnes de la nouvelle image");
        free(imageD);
        return NULL;
    }

    // Effectuer l'opration XOR
    int i ;
    for (i = 0; i < imageD->hauteur * imageD->largeur; i++) {
        imageD->image[i] = imageS->image[i] ^ key;
    }

    return imageD;
}

// Fonction pour crire une image dans un fichier .raw
// Paramtres :
//   - image : pointeur vers la structure IMAGE de l'image  crire
//   - filename : nom du fichier dans lequel crire l'image
void ecrire_image(IMAGE *image, char *filename) {
    // Ouvrir le fichier en mode criture binaire
    FILE *file = fopen(filename, "wb");
    if (file == NULL) {
        perror("Impossible d'ouvrir le fichier en criture");
        return;
    }

    // crire les dimensions de l'image
    fwrite(&(image->hauteur), sizeof(int), 1, file);
    fwrite(&(image->largeur), sizeof(int), 1, file);

    // crire les donnes de l'image
    fwrite(image->image, sizeof(UCHAR), image->hauteur * image->largeur, file);

    // Fermer le fichier
    fclose(file);
}

int main() {
    // Lire l'image
    IMAGE *imageS = lire_image("benji.raw");
    if (imageS == NULL) {
        return EXIT_FAILURE;
    }

    // Crypter/Decrypter l'image
    IMAGE *imageD = crypter_decrypter(imageS, 0xFF);
    if (imageD == NULL) {
        free(imageS->image);
        free(imageS);
        return EXIT_FAILURE;
    }

    // crire la nouvelle image dans un fichier
    ecrire_image(imageD, "image_cryptee.raw");

    // Librer la mmoire
    free(imageS->image);
    free(imageS);
    free(imageD->image);
    free(imageD);

    return 0;
}

