#include <stdio.h>
#include <stdlib.h>
#include <time.h>

//Nom:KEYANFE DONGMO ANASTASIE ANGE
//code:KEYA87330300
//Numero du groupe: 01
typedef unsigned char UCHAR;

struct IMAGERAW {
    int hauteur;
    int largeur;
    UCHAR *image;
};

typedef struct IMAGERAW IMAGE;

IMAGE *lire_image(const char *nom_fichier) {
    FILE *fichier = fopen(nom_fichier, "rb");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        exit(EXIT_FAILURE);
    }

    IMAGE *image = (IMAGE *)malloc(sizeof(IMAGE));
    if (image == NULL) {
        perror("Erreur lors de l'allocation de mmoire pour l'image");
        fclose(fichier);
        exit(EXIT_FAILURE);
    }

    fread(&(image->hauteur), sizeof(int), 1, fichier);
    fread(&(image->largeur), sizeof(int), 1, fichier);

    image->image = (UCHAR *)malloc(image->hauteur * image->largeur);
    if (image->image == NULL) {
        perror("Erreur lors de l'allocation de mmoire pour les pixels de l'image");
        fclose(fichier);
        free(image);
        exit(EXIT_FAILURE);
    }

    fread(image->image, sizeof(UCHAR), image->hauteur * image->largeur, fichier);

    fclose(fichier);
    return image;
}

void ecrire_image(const char *nom_fichier, IMAGE *image) {
    FILE *fichier = fopen(nom_fichier, "wb");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        exit(EXIT_FAILURE);
    }

    fwrite(&(image->hauteur), sizeof(int), 1, fichier);
    fwrite(&(image->largeur), sizeof(int), 1, fichier);
    fwrite(image->image, sizeof(UCHAR), image->hauteur * image->largeur, fichier);

    fclose(fichier);
}

IMAGE *crypter_decrypter(IMAGE *image, UCHAR cle, int par_colonne) {
    IMAGE *nouvelle_image = (IMAGE *)malloc(sizeof(IMAGE));
    if (nouvelle_image == NULL) {
        perror("Erreur lors de l'allocation de mmoire pour la nouvelle image");
        free(image->image);
        free(image);
        exit(EXIT_FAILURE);
    }

    nouvelle_image->hauteur = image->hauteur;
    nouvelle_image->largeur = image->largeur;

    nouvelle_image->image = (UCHAR *)malloc(nouvelle_image->hauteur * nouvelle_image->largeur);
    if (nouvelle_image->image == NULL) {
        perror("Erreur lors de l'allocation de mmoire pour les pixels de la nouvelle image");
        free(image->image);
        free(image);
        free(nouvelle_image);
        exit(EXIT_FAILURE);
    }

    int i, j;
    for (i = 0; i < image->hauteur; i++) {
        for (j = 0; j < image->largeur; j++) {
            int indice;
            if (par_colonne) {
                indice = j * image->hauteur + i;
            } else {
                indice = i * image->largeur + j;
            }
            nouvelle_image->image[indice] = image->image[indice] ^ cle;
        }
    }

    return nouvelle_image;
}

int main() {
    IMAGE *image_source, *image_cryptee;
    int choix;
    UCHAR cle;
    clock_t debut, fin;

    image_source = lire_image("benji.raw");

    printf("Entrez la cl de cryptage (8 bits) : ");
    scanf("%hhu", &cle);
    printf("Choisissez le mode (1 pour par range, 0 pour par colonne) : ");
    scanf("%d", &choix);

    debut = clock();

    image_cryptee = crypter_decrypter(image_source, cle, choix);

    fin = clock();
    double temps_execution = ((double)fin - debut) / CLOCKS_PER_SEC;
    printf("Temps d'excution : %f secondes\n", temps_execution);

    ecrire_image("image_cryptee.raw", image_cryptee);

    free(image_source->image);
    free(image_source);
    free(image_cryptee->image);
    free(image_cryptee);

    return 0;
}
